program moon(input,output);
{Determines fuel ratios for crewed Lunar mission.
 Program author: Steven S. Pietrobon, 21 Nov 1997.
 Revised: 20 Apr 1999, 17 Aug 2006.}

const nmax = 6; {array size}
      nmaxplus1 = 7;
      gFB = 0.0517; {fuel boiloff rate}
      gOB = 0.0063; {oxidiser boiloff rate}
      dv1 = 1578; {m/s, LOI delta V}
      dv2 = 1903; {m/s, LD delta V}
      dv3 = 1868; {m/s, LA delta V}
      dv4 =  880; {m/s, TEI delta V}
      dF =  70.9; {kg/kL, fuel density}
      dOX = 1149; {kg/kL, oxidiser density}
      neng = 6; {no. of parameters}
      error = 0.01;

type ve2ctor = array[1..nmax] of double;
     matrix = array[1..nmaxplus1,1..nmaxplus1] of double;

var A:matrix;
    mF:ve2ctor;
    I,J,K,Imin,Jmin,option,n,nFT1,nFT2,nOT1,nOT2:integer;
    gF1,gF2,gO1,gO2,R1,R2,me,ve1,ve2,mFT1,mFT2,mOT1,mOT2,mT:double;
    mFE,mOE,mOM,mFM,m,mmin,mF1,mF2,mO1,mO2,Dmax,Vmin,Amin:double;
    ans,ame:char;
    engine:text;
    Ra,ve2a:array[1..neng] of double;
    ma:array[1..neng,1..neng] of double;

function solve2(var a:matrix):ve2ctor;
var i,j,k,p:integer;
    m:double;
    x:ve2ctor;
begin{solve2}
  for i := 1 to n do
    begin{elimination}
      p := i;
      while (a[p,i] = 0.0) and (p < n+1) do p := p+1;
      if p = n+1 then
        begin{no solution}
          writeln('no unique solution exists');
          halt;
        end;{no solution}

      if p > i then
        for j := 1 to n+1 do
          begin{swap rows}
            m := a[i,j];
            a[i,j] := a[p,j];
            a[p,j] := m;
          end;{swap rows}

      for j := i+1 to n do
        begin{substitution}
          m := a[j,i]/a[i,i];
          for k := 1 to n+1 do
            a[j,k] := a[j,k] - m*a[i,k];
        end;{substitution}
    end;{elimination}

  for i := n downto 1 do
    begin{backward substitution}
      m := 0;
      for j := i+1 to n do
        m := m + a[i,j]*x[j];
      x[i] := (a[i,n+1] - m)/a[i,i];
    end;{backward substitution}

  solve2 := x;
end;{solve2}

function max(a,b:double):double;
begin{max}
  if a > b
    then max := a
    else max := b;
end;{max}

procedure write_data;
{Write data}
var LFT1,LFT2,LOT1,LOT2:double;
begin{write data}
  mFT1 := gF1*mF1;
  mFT2 := gF2*mF2;
  mOT1 := gO1*mO1;
  mOT2 := gO2*mO2;
  mT := mFT1+mFT2+mOT1+mOT2;

  if nFT1 = 0
    then LFT1 := -Dmax
    else LFT1 := max((mF1/(nFT1*dF) - Vmin)/(pi*sqr(Dmax/2)),0);
  LFT2 := max((mF2/(nFT2*dF) - Vmin)/(pi*sqr(Dmax/2)),0);
  if nOT1 = 0
    then LOT1 := -Dmax
    else LOT1 := max((mO1/(nOT1*dOX) - Vmin)/(pi*sqr(Dmax/2)),0);
  LOT2 := max((mO2/(nOT2*dOX) - Vmin)/(pi*sqr(Dmax/2)),0);
  write('Fuel tank mass 1      = ',mFT1:7:1,' kg, L = ',(Dmax+LFT1):4:2,' m');
  writeln(', n = ',nFT1:1);
  write('Oxidiser tank mass 1  = ',mOT1:7:1,' kg, L = ',(Dmax+LOT1):4:2,' m');
  writeln(', n = ',nOT1:1);
  write('Fuel tank mass 2      = ',mFT2:7:1,' kg, L = ',(Dmax+LFT2):4:2,' m');
  writeln(', n = ',nFT2:1);
  write('Oxidiser tank mass 2  = ',mOT2:7:1,' kg, L = ',(Dmax+LOT2):4:2,' m');
  writeln(', n = ',nOT2:1);
  writeln;
  writeln('Fuel mass to Moon     = ',mFE:7:1,' kg');
  writeln('Fuel made on Moon     = ',mFM:7:1,' kg');
  writeln('Oxidiser mass to Moon = ',mOE:7:1,' kg');
  writeln('Oxidiser made on Moon = ',mOM:7:1,' kg');
  writeln('Tank mass             = ',mT:7:1,' kg');
  writeln('Payload mass          = ',me:7:1,' kg');
  writeln('TEI mass              = ',m:7:1,' kg');

  if nFT1 > 0
    then begin{fuel tank 1}
           mFT1 := nFT1*(32.3*exp(0.795*ln(mF1/(dF*nFT1)))
                         + 1.77*(Amin+pi*Dmax*LFT1));
           gF1 := mFT1/mF1;
         end{fuel tank 1}
    else gF1 := 0;
  mFT2 := nFT2*(32.3*exp(0.795*ln(mF2/(dF*nFT2))) + 1.77*(Amin+pi*Dmax*LFT2));
  gF2 := mFT2/mF2;

  if nOT1 > 0
    then begin{oxidiser tank 1}
           mOT1 := nOT1*(27.0*exp(0.843*ln(mO1/(dOX*nOT1)))
                         + 2.48*(Amin+pi*Dmax*LOT2));
           gO1 := mOT1/mO1;
         end{oxidiser tank 1}
    else gO1 := 0;
  mOT2 := nOT2*(27.0*exp(0.843*ln(mO2/(dOX*nOT2))) + 2.48*(Amin+pi*Dmax*LOT2));
  gO2 := mOT2/mO2;
end;{write data}

procedure option1;
{Direct Ascent}
begin{option 1}
  nFT1 := 4;
  nFT2 := 2;
  nOT1 := 2;
  nOT2 := 1;
  if ame = 'y'
    then n := 2
    else n := 3;
  repeat
    A[1,1] := (R1+1)/(exp((dv1+dv2)/ve1)-1) - (gF1+gO1*R1);
    A[1,2] := -((1+gF2)*(1+gFB)+(1+gO2)*R2*(1+gOB));

    A[2,1] := 0;
    A[2,2] := (R2+1)/(exp((dv3+dv4)/ve2)-1) - (gF2*(1+gFB)+gO2*R2*(1+gOB));

    if ame = 'y'
      then begin{me}
             for K := 1 to n do
               A[K,n+1] := me;
           end{me}
      else begin{m}
             A[3,1] := 1+gF1+(1+gO1)*R1;
             A[3,2] := -A[1,2];
             A[3,3] := 1;
             A[3,4] := m;

             for K := 1 to n-1 do
               begin{m init}
                 A[K,n] := -1;
                 A[K,n+1] := 0;
               end;{m init}
           end;{m}

    mF := solve2(A);
    writeln;
    writeln('First ratio = ',R1:3:1,', Second ratio = ',R2:3:1);
    write('Fuel masses (kg):     ');
    if ame = 'y'
      then for K := 1 to n do write(mF[K]:7:1,' ')
      else for K := 1 to n-1 do write(mF[K]:7:1,' ');
    writeln(gFB*mF[2]:7:1);
    write('Oxidiser masses (kg): ');
    write((R1*mF[1]):7:1,' ');
    write((R2*mF[2]):7:1,' ');
    writeln(gOB*R2*mF[2]:7:1);
    mF1 := mF[1];
    mF2 := (1+gFB)*mF[2];
    mO1 := R1*mF[1];
    mO2 := R2*(1+gOB)*mF[2];
    mFE := mF1+mF2;
    mFM := 0;
    mOE := mO1+mO2;
    mOM := 0;
    if ame = 'y' 
      then m := me+mFE+mOE+mT
      else me := mF[n];
    write_data;
  until abs(mFT1+mFT2+mOT1+mOT2-mT) < error;
end;{option 1}

procedure option2;
{Lunar Surface Propellant Transfer (O2)}
begin{option 2}
  nFT1 := 4;
  nFT2 := 2;
  nOT1 := 0;
  nOT2 := 2;
  if ame = 'y'
    then n := 2
    else n := 3;
  repeat
    A[1,1] := (R1+1)/(exp((dv1+dv2)/ve1)-1) - (gF1+gO2*R1);
    A[1,2] := -(1+gF2)*(1+gFB);

    A[2,1] := -gO2*R1;
    A[2,2] := (R2+1)/(exp((dv3+dv4)/ve2)-1) - gF2*(1+gFB);

    if ame = 'y'
      then begin{me}
             for K := 1 to n do
               A[K,n+1] := me;
           end{me}
      else begin{m}
             A[3,1] := 1+gF1+(1+gO2)*R1;
             A[3,2] := -A[1,2];
             A[3,3] := 1;
             A[3,4] := m;

             for K := 1 to n-1 do
               begin{m init}
                 A[K,n] := -1;
                 A[K,n+1] := 0;
               end;{m init}
           end;{m}

    mF := solve2(A);
    writeln;
    writeln('First Ratio ratio = ',R1:3:1,', Second ratio = ',R2:3:1);
    write('Fuel masses (kg):     ');
    if ame = 'y'
      then for K := 1 to n do write(mF[K]:7:1,' ')
      else for K := 1 to n-1 do write(mF[K]:7:1,' ');
    writeln(gFB*mF[2]:7:1);
    write('Oxidiser masses (kg): ');
    write((R1*mF[1]):7:1,' ');
    write((R2*mF[2]):7:1,' ');
    writeln(0.0:7:1);
    mF1 := mF[1];
    mF2 := (1+gFB)*mF[2];
    mO1 := 0;
    mO2 := mF[1]*R1;
    mFE := mF1+mF2;
    mFM := 0;
    mOE := mO2;
    mOM := R2*mF[2];
    if ame = 'y' 
      then m := me+mFE+mOE+mT
      else me := mF[n];
    write_data;
  until abs(mFT1+mFT2+mOT1+mOT2-mT) < error;
end;{option 2}

procedure option3;
{Lunar Orbit Propellant Transfer (O2)}
begin{option 3}
  nFT1 := 4;
  nFT2 := 2;
  nOT1 := 0;
  nOT2 := 2;
  if ame = 'y'
    then n := 5
    else n := 6;
  repeat
    A[1,1] := (R1+1)/(exp(dv1/ve1)-1) - gF1;
    A[1,2] := -(1+gF1+gO2*R2);
    A[1,3] := -((1+gF2)*(1+gFB)+gO2*R2);
    A[1,4] := A[1,3];
    A[1,5] := (gO2-1)*R1;

    A[2,1] := -gF1;
    A[2,2] := (R2+1)/(exp(dv2/ve2)-1) - (gF1+gO2*R2);
    A[2,3] := A[1,3];
    A[2,4] := A[1,3];
    A[2,5] := gO2*R1;

    A[3,1] := 0;
    A[3,2] := -(1+gO2)*R2;
    A[3,3] := (R2+1)/(exp(dv3/ve2)-1) - (gF2*(1+gFB)+gO2*R2);
    A[3,4] := -(1+gF2*(1+gFB)+(1+gO2)*R2);
    A[3,5] := (1+gO2)*R1;

    A[4,1] := 0;
    A[4,2] := -gO2*R2;
    A[4,3] := -(gF2*(1+gFB)+gO2*R2);
    A[4,4] := (R2+1)/(exp(dv4/ve2)-1) + A[4,3];
    A[4,5] := gO2*R1;

    A[5,1] := 0;
    A[5,2] := -gO2*R2;
    A[5,3] := A[4,3];
    A[5,4] := A[4,3];
    A[5,5] := (R1+1)/(exp(dv4/ve1)-1) + gO2*R1;

    if ame = 'y'
      then begin{me}
             for K := 1 to n do
               A[K,n+1] := me;
           end{me}
      else begin{m}
             A[6,1] := 1+gF1+R1;
             for K := 2 to n-1 do
               A[6,K] := -A[1,K];
             A[6,6] := 1;
             A[6,7] := m;

             for K := 1 to n-1 do
               begin{m init}
                 A[K,n] := -1;
                 A[K,n+1] := 0;
               end;{m init}
           end;{m}

    mF := solve2(A);
    writeln;
    writeln('First ratio = ',R1:3:1,', Second ratio = ',R2:3:1);
    write('Fuel masses (kg):     ');
    if ame = 'y'
      then for K := 1 to n do write(mF[K]:7:1,' ')
      else for K := 1 to n-1 do write(mF[K]:7:1,' ');
    writeln(gFB*(mF[3]+mF[4]):7:1);
    write('Oxidiser masses (kg): ');
    write((R1*mF[1]):7:1,' ');
    write((R2*mF[2]):7:1,' ');
    write((R2*mF[3]):7:1,' ');
    write((R2*mF[4]):7:1,' ');
    write((R1*mF[5]):7:1,' ');
    writeln(0.0:7:1);
    mF1 := mF[1]+mF[2];
    mF2 := (1+gFB)*(mF[3]+mF[4]);
    mO1 := 0;
    mO2 := R2*(mF[2]+mF[3]+mF[4])-R1*mF[5];
    mFE := mF1+mF2;
    mFM := 0;
    mOE := R1*(mF[1]+mF[5]);
    mOM := mO2;
    if ame = 'y'
      then m := me+mFE+mOE+mT
      else me := mF[n];
    write_data;
  until abs(mFT1+mFT2+mOT1+mOT2-mT) < error;
end;{option 3}

procedure option4;
{Lunar Surface Propellant Transfer (O2/H2)}
begin{option 4}
  nFT1 := 0;
  nFT2 := 4;
  nOT1 := 0;
  nOT2 := 2;
  if ame = 'y'
    then n := 2
    else n := 3;
  repeat
    A[1,1] := (R1+1)/(exp((dv1+dv2)/ve1)-1) - (gF2+gO2*R1);
    A[1,2] := 0;

    A[2,1] := -(gF2+gO2*R1);
    A[2,2] := (R2+1)/(exp((dv3+dv4)/ve2)-1);

    if ame = 'y'
      then begin{me}
             for K := 1 to n do
               A[K,n+1] := me;
           end{me}
      else begin{m}
             A[3,1] := 1+gF2+(1+gO2)*R1;
             A[3,2] := 0;
             A[3,3] := 1;
             A[3,4] := m;

             for K := 1 to n-1 do
               begin{m init}
                 A[K,n] := -1;
                 A[K,n+1] := 0;
               end;{m init}
           end;{m}

    mF := solve2(A);
    writeln;
    writeln('First Ratio ratio = ',R1:3:1,', Second ratio = ',R2:3:1);
    write('Fuel masses (kg):     ');
    if ame = 'y'
      then for K := 1 to n do write(mF[K]:7:1,' ')
      else for K := 1 to n-1 do write(mF[K]:7:1,' ');
    writeln(0.0:7:1);
    write('Oxidiser masses (kg): ');
    write((R1*mF[1]):7:1,' ');
    write((R2*mF[2]):7:1,' ');
    writeln(0.0:7:1);
    mF1 := 0;
    mF2 := mF[1];
    mO1 := 0;
    mO2 := R1*mF[1];
    mFE := mF[1];
    mFM := mF[2];
    mOE := mO2;
    mOM := R2*mF[2];
    if ame = 'y' 
      then m := me+mFE+mOE+mT
      else me := mF[n];
    write_data;
  until abs(mFT1+mFT2+mOT1+mOT2-mT) < error;
end;{option 4}

procedure option5;
{Lunar Orbit Propellant Transfer (O2/H2)}
begin{option 5}
  nFT1 := 0;
  nFT2 := 4;
  nOT1 := 0;
  nOT2 := 2;
  if ame = 'y'
    then n := 5
    else n := 6;
  repeat
    A[1,1] := (R1+1)/(exp(dv1/ve1)-1) - gF2;
    A[1,2] := -(1+gF2+gO2*R2);
    A[1,3] := -gO2*R2;
    A[1,4] := A[1,3];
    A[1,5] := (gO2-1)*R1;

    A[2,1] := -gF2;
    A[2,2] := (R2+1)/(exp(dv2/ve2)-1) - (gF2+gO2*R2);
    A[2,3] := A[1,3];
    A[2,4] := A[1,3];
    A[2,5] := gO2*R1;

    A[3,1] := -gF2;
    A[3,2] := -(gF2+(1+gO2)*R2);
    A[3,3] := (R2+1)/(exp(dv3/ve2)-1) + A[1,3];
    A[3,4] := -(1+gO2)*R2;
    A[3,5] := (1+gO2)*R1;

    A[4,1] := -gF2;
    A[4,2] := -(gF2+gO2*R2);
    A[4,3] := A[1,3];
    A[4,4] := (R2+1)/(exp(dv4/ve2)-1) + A[1,3];
    A[4,5] := gO2*R1;

    A[5,1] := -gF2;
    A[5,2] := -(gF2+gO2*R2);
    A[5,3] := A[1,3];
    A[5,4] := A[1,3];
    A[5,5] := (R1+1)/(exp(dv4/ve1)-1) + gO2*R1;

    if ame = 'y'
      then begin{me}
             for K := 1 to n do
               A[K,n+1] := me;
           end{me}
      else begin{m}
             A[6,1] := 1+gF2+R1;
             for K := 2 to n-1 do
               A[6,K] := -A[1,K];
             A[6,6] := 1;
             A[6,7] := m;

             for K := 1 to n-1 do
               begin{m init}
                 A[K,n] := -1;
                 A[K,n+1] := 0;
               end;{m init}
           end;{m}

    mF := solve2(A);
    writeln;
    writeln('First ratio = ',R1:3:1,', Second ratio = ',R2:3:1);
    write('Fuel masses (kg):     ');
    if ame = 'y'
      then for K := 1 to n do write(mF[K]:7:1,' ')
      else for K := 1 to n-1 do write(mF[K]:7:1,' ');
    writeln(0.0:7:1);
    write('Oxidiser masses (kg): ');
    write((R1*mF[1]):7:1,' ');
    write((R2*mF[2]):7:1,' ');
    write((R2*mF[3]):7:1,' ');
    write((R2*mF[4]):7:1,' ');
    write((R1*mF[5]):7:1,' ');
    writeln(0.0:7:1);
    mF1 := 0;
    mF2 := mF[1]+mF[2];
    mO1 := 0;
    mO2 := R2*(mF[2]+mF[3]+mF[4])-R1*mF[5];
    mFE := mF[1]+mF[2];
    mFM := mF[3]+mF[4];
    mOE := R1*(mF[1]+mF[5]);
    mOM := mO2;
    if ame = 'y' 
      then m := me+mFE+mOE+mT
      else me := mF[n];
    write_data;
  until abs(mFT1+mFT2+mOT1+mOT2-mT) < error;
end;{option 5}

begin{main}
  assign(engine,'engine.dat');
  reset(engine);
  for I := 1 to neng do
    begin{read engine}
      read(engine,Ra[I]);
      readln(engine,ve2a[I]);
    end;{read engine}
  close(engine);

  writeln('1 = Direct Ascent');
  writeln('2 = Lunar Surface Propellant Transfer (O2)');
  writeln('3 = Lunar Orbit Propellant Transfer (O2)');
  writeln('4 = Lunar Surface Propellant Transfer (O2/H2)');
  writeln('5 = Lunar Orbit Propellant Transfer (O2/H2)');
  write('Enter option: ');
  readln(option);
  write('Enter maximum propellant tank diameter (m) ');
  readln(Dmax);
  Amin := pi*sqr(Dmax);
  Vmin := Amin*Dmax/6;

  write('Constant payload mass? ');
  readln(ame);
  if ame = 'Y' then ame := 'y';
  if ame = 'y'
    then begin{me}
           write('Enter payload mass (kg): ');
           readln(me);
         end{me}
    else begin{m}
           write('Enter trans Lunar injection mass (kg): ');
           readln(m);
         end;{m}

  gF1 := 0.5;
  gF2 := 0.5;
  gO1 := 0.03;
  gO2 := 0.03;

  write('Optimise mixture ratios? ');
  readln(ans);
  if (ans = 'y') or (ans = 'Y')
    then begin{optimise}
           if ame = 'y'
             then mmin := 999999
             else mmin := 0;
           for I := 1 to neng do
           begin{vary R1}
             R1 := Ra[I];
             ve1 := ve2a[I];
             for J := 1 to neng do
               begin{vary R2}
                 R2 := Ra[J];
                 ve2 := ve2a[J];
                 case option of
                   1: option1;
                   2: option2;
                   3: option3;
                   4: option4;
                   5: option5;
                 end;
                 if ame = 'y'
                   then begin{me}
                          if m < mmin then 
                            begin{new min}
                              Imin := I;
                              Jmin := J;
                              mmin := m;
                            end;{new min}
                          ma[I,J] := m;
                        end{me}
                   else begin{m}
                          if me > mmin then 
                            begin{new min}
                              Imin := I;
                              Jmin := J;
                              mmin := me;
                            end;{new min}
                          ma[I,J] := me;
                        end;{me}
               end;{vary R2}
           end;{vary R1}
       end{optimise}
    else begin{enter ratios}
           write('Enter first and second mixture ratios: ');
           readln(R1,R2);
           Imin := round(2*R1)-9;
           Jmin := round(2*R2)-9;
         end;{enter ratios}

  R1 := Ra[Imin];
  ve1 := ve2a[Imin];
  R2 := Ra[Jmin];
  ve2 := ve2a[Jmin];
  case option of
    1: option1;
    2: option2;
    3: option3;
    4: option4;
    5: option5;
  end;
  if (ans = 'y') or (ans = 'Y') then
    begin{write table}
      writeln;
      for I := 1 to neng do
         begin{vary R1}
           for J := 1 to neng do
             write(ma[I,J]:7:1,' ');
           writeln;
         end;{vary R1}
    end;{write table}
end.{main}